@@ -1,6 +1,7 @@
 /*
  *   SPDX-FileCopyrightText: 2012 Aleix Pol Gonzalez <aleixpol@blue-systems.com>
  *   SPDX-FileCopyrightText: 2013 Lukas Appelhans <l.appelhans@gmx.de>
+ *   SPDX-FileCopyrightText: 2023 Harald Sitter <sitter@kde.org>
  *
  *   SPDX-License-Identifier: GPL-2.0-only OR GPL-3.0-only OR LicenseRef-KDE-Accepted-GPL
  */
@@ -82,6 +83,17 @@ QString PackageKitBackend::locateService(const QString &filename)
     return QStandardPaths::locate(QStandardPaths::GenericDataLocation, QStringLiteral("applications/") + filename);
 }
 
+Delay::Delay()
+{
+    m_delay.setSingleShot(true);
+    m_delay.setInterval(100);
+
+    connect(&m_delay, &QTimer::timeout, this, [this] {
+        Q_EMIT perform(m_pkgids);
+        m_pkgids.clear();
+    });
+}
+
 PackageKitBackend::PackageKitBackend(QObject *parent)
     : AbstractResourcesBackend(parent)
     , m_appdata(new AppStream::Pool)
@@ -96,9 +108,56 @@ PackageKitBackend::PackageKitBackend(QObject *parent)
     t->setSingleShot(false);
     t->start();
 
-    m_delayedDetailsFetch.setSingleShot(true);
-    m_delayedDetailsFetch.setInterval(100);
-    connect(&m_delayedDetailsFetch, &QTimer::timeout, this, &PackageKitBackend::performDetailsFetch);
+    connect(&m_details, &Delay::perform, this, &PackageKitBackend::performDetailsFetch);
+    connect(&m_details, &Delay::perform, this, [this](const QSet<QString> &pkgids) {
+        PackageKit::Transaction *t = PackageKit::Daemon::getUpdatesDetails(kSetToList(pkgids));
+        connect(t,
+                &PackageKit::Transaction::updateDetail,
+                this,
+                [this](const QString &packageID,
+                       const QStringList &updates,
+                       const QStringList &obsoletes,
+                       const QStringList &vendorUrls,
+                       const QStringList &bugzillaUrls,
+                       const QStringList &cveUrls,
+                       PackageKit::Transaction::Restart restart,
+                       const QString &updateText,
+                       const QString &changelog,
+                       PackageKit::Transaction::UpdateState state,
+                       const QDateTime &issued,
+                       const QDateTime &updated) {
+                    const QSet<AbstractResource *> resources = resourcesByPackageName(PackageKit::Daemon::packageName(packageID));
+                    for (auto r : resources) {
+                        PackageKitResource *resource = qobject_cast<PackageKitResource *>(r);
+                        if (resource->containsPackageId(packageID)) {
+                            resource->updateDetail(packageID,
+                                                   updates,
+                                                   obsoletes,
+                                                   vendorUrls,
+                                                   bugzillaUrls,
+                                                   cveUrls,
+                                                   restart,
+                                                   updateText,
+                                                   changelog,
+                                                   state,
+                                                   issued,
+                                                   updated);
+                        }
+                    }
+                });
+        connect(t, &PackageKit::Transaction::errorCode, this, [this, pkgids](PackageKit::Transaction::Error err, const QString &error) {
+            qWarning() << "error fetching updates:" << err << error;
+            for (const QString &pkgid : pkgids) {
+                const QSet<AbstractResource *> resources = resourcesByPackageName(PackageKit::Daemon::packageName(pkgid));
+                for (auto r : resources) {
+                    PackageKitResource *resource = qobject_cast<PackageKitResource *>(r);
+                    if (resource->containsPackageId(pkgid)) {
+                        Q_EMIT resource->changelogFetched(QString());
+                    }
+                }
+            }
+        });
+    });
 
     connect(PackageKit::Daemon::global(), &PackageKit::Daemon::restartScheduled, m_updater, &PackageKitUpdater::enableNeedsReboot);
     connect(PackageKit::Daemon::global(), &PackageKit::Daemon::isRunningChanged, this, &PackageKitBackend::checkDaemonRunning);
@@ -502,7 +561,7 @@ static const auto needsResolveFilter = [](AbstractResource *res) {
 
 class PKResultsStream : public ResultsStream
 {
-public:
+private:
     PKResultsStream(PackageKitBackend *backend, const QString &name)
         : ResultsStream(name)
         , backend(backend)
@@ -518,6 +577,13 @@ public:
         });
     }
 
+public:
+    template<typename... Args>
+    [[nodiscard]] static QPointer<PKResultsStream> create(Args&& ...args)
+    {
+        return new PKResultsStream(std::forward<Args>(args)...);
+    }
+
     void sendResources(const QVector<AbstractResource *> &res, bool waitForResolved = false)
     {
         if (res.isEmpty()) {
@@ -554,8 +620,11 @@ ResultsStream *PackageKitBackend::search(const AbstractResourcesBackend::Filters
     if (!filter.resourceUrl.isEmpty()) {
         return findResourceByPackageName(filter.resourceUrl);
     } else if (!filter.extends.isEmpty()) {
-        auto stream = new PKResultsStream(this, QStringLiteral("PackageKitStream-extends"));
+        auto stream = PKResultsStream::create(this, QStringLiteral("PackageKitStream-extends"));
         auto f = [this, filter, stream] {
+            if (!stream) {
+                return;
+            }
             const auto resources = kTransform<QVector<AbstractResource *>>(m_packages.extendedBy.value(filter.extends), [](AppPackageKitResource *a) {
                 return a;
             });
@@ -567,8 +636,11 @@ ResultsStream *PackageKitBackend::search(const AbstractResourcesBackend::Filters
         return new ResultsStream(QStringLiteral("PackageKitStream-upgradeable"),
                                  kTransform<QVector<AbstractResource *>>(upgradeablePackages())); // No need for it to be a PKResultsStream
     } else if (filter.state == AbstractResource::Installed) {
-        auto stream = new PKResultsStream(this, QStringLiteral("PackageKitStream-installed"));
+        auto stream = PKResultsStream::create(this, QStringLiteral("PackageKitStream-installed"));
         auto f = [this, stream, filter] {
+            if (!stream) {
+                return;
+            }
             const auto toResolve = kFilter<QVector<AbstractResource *>>(m_packages.packages, needsResolveFilter);
 
             auto installedAndNameFilter = [filter](AbstractResource *res) {
@@ -607,8 +679,11 @@ ResultsStream *PackageKitBackend::search(const AbstractResourcesBackend::Filters
         runWhenInitialized(f, stream);
         return stream;
     } else if (filter.search.isEmpty() && !filter.category) {
-        auto stream = new PKResultsStream(this, QStringLiteral("PackageKitStream-all"));
+        auto stream = PKResultsStream::create(this, QStringLiteral("PackageKitStream-all"));
         auto f = [this, filter, stream] {
+            if (!stream) {
+                return;
+            }
             auto resources = kFilter<QVector<AbstractResource *>>(m_packages.packages, [](AbstractResource *res) {
                 return res->type() != AbstractResource::Technical && !qobject_cast<PackageKitResource *>(res)->isCritical()
                     && !qobject_cast<PackageKitResource *>(res)->extendsItself();
@@ -618,8 +693,11 @@ ResultsStream *PackageKitBackend::search(const AbstractResourcesBackend::Filters
         runWhenInitialized(f, stream);
         return stream;
     } else {
-        auto stream = new PKResultsStream(this, QStringLiteral("PackageKitStream-search"));
+        auto stream = PKResultsStream::create(this, QStringLiteral("PackageKitStream-search"));
         const auto f = [this, stream, filter]() {
+            if (!stream) {
+                return;
+            }
             const auto components = !filter.search.isEmpty() ? m_appdata->search(filter.search)
 #if ASQ_CHECK_VERSION(0, 15, 6)
                                   : filter.category          ? AppStreamUtils::componentsByCategories(m_appdata.get(),
@@ -649,7 +727,7 @@ void PackageKitBackend::runWhenInitialized(const std::function<void()> &f, QObje
     if (!m_appstreamInitialized) {
         connect(this, &PackageKitBackend::loadedAppStream, stream, f);
     } else {
-        QTimer::singleShot(0, this, f);
+        QTimer::singleShot(0, stream, f); // NOTE `stream` is a child of `this` so this transitively also depends on `this`
     }
 }
 
@@ -663,15 +741,18 @@ PKResultsStream *PackageKitBackend::findResourceByPackageName(const QUrl &url)
             || mime.inherits(QStringLiteral("application/x-tar")) //
             || mime.inherits(QStringLiteral("application/x-zstd-compressed-tar")) //
             || mime.inherits(QStringLiteral("application/x-xz-compressed-tar"))) {
-            return new PKResultsStream(this, QStringLiteral("PackageKitStream-localpkg"), {new LocalFilePKResource(url, this)});
+            return PKResultsStream::create(this, QStringLiteral("PackageKitStream-localpkg"), QVector<AbstractResource *>{new LocalFilePKResource(url, this)}).data();
         }
     } else if (url.scheme() == QLatin1String("appstream")) {
         const auto appstreamIds = AppStreamUtils::appstreamIds(url);
         if (appstreamIds.isEmpty())
             Q_EMIT passiveMessage(i18n("Malformed appstream url '%1'", url.toDisplayString()));
         else {
-            auto stream = new PKResultsStream(this, QStringLiteral("PackageKitStream-appstream-url"));
+            auto stream = PKResultsStream::create(this, QStringLiteral("PackageKitStream-appstream-url"));
             const auto f = [this, appstreamIds, stream]() {
+                if (!stream) {
+                    return;
+                }
                 auto toSend = QSet<AbstractResource *>();
                 toSend.reserve(appstreamIds.size());
                 for (const auto &appstreamId : appstreamIds) {
@@ -690,7 +771,7 @@ PKResultsStream *PackageKitBackend::findResourceByPackageName(const QUrl &url)
             return stream;
         }
     }
-    return new PKResultsStream(this, QStringLiteral("PackageKitStream-unknown-url"), {});
+    return PKResultsStream::create(this, QStringLiteral("PackageKitStream-unknown-url"), QVector<AbstractResource *>{}).data();
 }
 
 template<typename T>
@@ -829,11 +910,6 @@ void PackageKitBackend::getUpdatesFinished(PackageKit::Transaction::Exit, uint)
         Q_EMIT updatesCountChanged();
 }
 
-bool PackageKitBackend::isPackageNameUpgradeable(const PackageKitResource *res) const
-{
-    return !upgradeablePackageId(res).isEmpty();
-}
-
 // Copy of Transaction::packageName that doesn't create a copy but just pass a reference
 // It's an optimisation as there's a bunch of allocations that happen from packageName
 // Having packageName return a QStringRef or a QStringView would fix this issue.
@@ -843,6 +919,16 @@ static QStringView TransactionpackageName(const QString &packageID)
     return QStringView(packageID).left(packageID.indexOf(QLatin1Char(';')));
 }
 
+bool PackageKitBackend::isPackageNameUpgradeable(const PackageKitResource *res) const
+{
+    const QString name = res->packageName();
+    for (const QString &pkgid : m_updatesPackageId) {
+        if (TransactionpackageName(pkgid) == name)
+            return true;
+    }
+    return false;
+}
+
 QSet<QString> PackageKitBackend::upgradeablePackageId(const PackageKitResource *res) const
 {
     QSet<QString> ids;
@@ -856,22 +942,17 @@ QSet<QString> PackageKitBackend::upgradeablePackageId(const PackageKitResource *
 
 void PackageKitBackend::fetchDetails(const QSet<QString> &pkgid)
 {
-    if (!m_delayedDetailsFetch.isActive()) {
-        m_delayedDetailsFetch.start();
-    }
-
-    m_packageNamesToFetchDetails += pkgid;
+    m_details.add(pkgid);
 }
 
-void PackageKitBackend::performDetailsFetch()
+void PackageKitBackend::performDetailsFetch(const QSet<QString> &pkgids)
 {
-    Q_ASSERT(!m_packageNamesToFetchDetails.isEmpty());
-    const auto ids = m_packageNamesToFetchDetails.values();
+    Q_ASSERT(!pkgids.isEmpty());
+    const auto ids = pkgids.values();
 
     PackageKit::Transaction *transaction = PackageKit::Daemon::getDetails(ids);
     connect(transaction, &PackageKit::Transaction::details, this, &PackageKitBackend::packageDetails);
     connect(transaction, &PackageKit::Transaction::errorCode, this, &PackageKitBackend::transactionError);
-    m_packageNamesToFetchDetails.clear();
 }
 
 void PackageKitBackend::checkDaemonRunning()
