@@ -3,20 +3,25 @@
  *
  *   SPDX-License-Identifier: GPL-2.0-only OR GPL-3.0-only OR LicenseRef-KDE-Accepted-GPL
  */
- 
-#ifndef OSTREERPMRESOURCE_H
-#define OSTREERPMRESOURCE_H
+
+#pragma once
+
+#include "OstreeFormat.h"
 
 #include <resources/AbstractResource.h>
 
 class RpmOstreeBackend;
 class QAbstractItemModel;
+
+/*
+ * Represents an ostree deployment (an installed version of the system) as a
+ * resource in Discover.
+ */
 class RpmOstreeResource : public AbstractResource
 {
     Q_OBJECT
-    Q_PROPERTY(QStringList objects MEMBER m_objects CONSTANT)
 public:
-    explicit RpmOstreeResource(const QVariantMap &, RpmOstreeBackend *);
+    RpmOstreeResource(const QVariantMap &map, RpmOstreeBackend *parent);
 
     QString appstreamId() const override;
     AbstractResource::State state() override;
@@ -33,15 +38,15 @@ public:
     QString installedVersion() const override;
     QString origin() const override;
     QString section() override;
-    void fetchScreenshots() override {};
+    void fetchScreenshots() override{};
     quint64 size() override;
     QString sizeDescription() override;
-    void fetchChangelog() override {};
+    void fetchChangelog() override{};
     QStringList extends() const override;
     AbstractResource::Type type() const override;
     QString author() const override;
     bool canExecute() const override;
-    void invokeApplication() const override {};
+    void invokeApplication() const override{};
     QUrl url() const override;
     QString sourceIcon() const override;
     QUrl homepage() override;
@@ -51,34 +56,39 @@ public:
     QUrl donationURL() override;
 
     void setState(AbstractResource::State);
-    void fetchRemoteRefs();
-    void setNewVersion(QString);
 
-    static const QStringList m_objects;
+    /* Get the current version */
+    QString version();
 
-    /*
-     * It is called when the user clicks on the button to switch to a new kinoite refs and
-     * it emits buttonPressed() so that the corresponding system upgrade function
-     * is executed in the backend.
-     */
-    Q_SCRIPTABLE void rebaseToNewVersion();
+    /* Set the target version for updates */
+    void setNewVersion(const QString &newVersion);
 
-    /** Returns the next major version for the current deployment. */
-    Q_SCRIPTABLE QString getNextMajorVersion();
+    /* Get the target version for updates */
+    QString getNewVersion() const;
 
-    /**
-     * Returns true only if there is a newer ref available in the current remote.
-     * TODO: Only display this button when the new version is stable (and not just branched as it works today.
-     */
-    Q_SCRIPTABLE bool isNextMajorVersionAvailable();
+    /* Validate and set the target major version for rebase */
+    bool setNewMajorVersion(const QString &newMajorVersion);
 
-    /** Returns if a given deployment is the currently booted deployment. */
+    /* Returns the next major version for the deployment */
+    QString getNextMajorVersion() const;
+
+    /* Returns the ostree ref for the next major version for the deployment */
+    QString getNextMajorVersionRef() const;
+
+    /* Returns if a given deployment is the currently booted deployment */
     Q_SCRIPTABLE bool isBooted();
 
-Q_SIGNALS:
+    /* Returns if a given deployment is currently pending */
+    Q_SCRIPTABLE bool isPending();
 
-    /** Signal emitted when the user requests the rebase to a newer version */
-    void buttonPressed(QString);
+    /* Returns true only if the deployment is from a classic Ostree format */
+    bool isClassic();
+
+    /* Returns true only if the deployment is from an OCI based format */
+    bool isOCI();
+
+    /* Get the full repo:tag reference, only if it's OCI */
+    QString OCIUrl();
 
 private:
     QString m_name;
@@ -86,23 +96,23 @@ private:
     QString m_osname;
     QString m_version;
     QDate m_timestamp;
-    QString m_origin;
-    QString m_remote;
-    QString m_branch;
-    QString m_branchName;
-    QString m_branchVersion;
-    QString m_branchArch;
-    QString m_branchVariant;
     QString m_appstreamid;
     bool m_booted;
     bool m_pinned;
+    bool m_pending;
+    QStringList m_requested_base_local_replacements;
+    QStringList m_requested_base_removals;
+    QStringList m_requested_local_packages;
+    QStringList m_requested_modules;
+    QStringList m_requested_packages;
+    QString m_checksum;
+
+    /* Store the format used by ostree to pull each deployment*/
+    QScopedPointer<OstreeFormat> m_ostreeFormat;
 
     AbstractResource::State m_state;
 
     QString m_newVersion;
     QString m_nextMajorVersion;
-    QStringList m_remoteRefs;
-    QString m_currentRefs;
+    QString m_nextMajorVersionRef;
 };
-
-#endif
